-- Puffin: A Tojaso FowlPlay Project
-- Load configuration data into the shared database

local _, Puffin = ...
local EF = Puffin.Error
local DB = Puffin.Database
local override = true -- set to false if using external options panel (requires merging updates)
local configurationMode = false -- set to true to display views to assist in configuration

-- Color defaults saved in the shared database
local defaultBlack = { r = 0, g = 0, b = 0, a = 1 }
local defaultDark = { r = 0.15, g = 0.15, b = 0.15, a = 1 }
local defaultGray = { r = 0.25, g = 0.25, b = 0.25, a = 1 }
local defaultLightGray = { r = 0.5, g = 0.5, b = 0.5, a = 1 }
local defaultLight = { r = 0.75, g = 0.75, b = 0.75, a = 1 }
local defaultWhite = { r = 1, g = 1, b = 1, a = 1 }
local defaultRed = { r = 1, g = 0, b = 0, a = 1 }
local defaultBlue = { r = 0, g = 0, b = 1, a = 1 }
local defaultLightBlue = { r = 0.45, g = 0.62, b = 0.81, a = 1 }
local defaultGreen = { r = 0, g = 1, b = 0, a = 1 }
local defaultYellow = { r = 1, g = 1, b = 0, a = 1 }
local defaultOrange = { r = 1, g = 0.6, b = 0, a = 1 }
local defaultTransparentBlack = { r = 0, g = 0, b = 0, a = 0.5 }
local defaultTransparentDark = { r = 0.15, g = 0.15, b = 0.15, a = 0.5 }
local defaultTransparentGray = { r = 0.25, g = 0.25, b = 0.25, a = 0.5 }
local defaultTransparentLightGray = { r = 0.5, g = 0.5, b = 0.5, a = 0.5 }
local defaultTransparentLight = { r = 0.75, g = 0.75, b = 0.75, a = 0.5 }
local defaultTransparentWhite = { r = 1, g = 1, b = 1, a = 0.5 }
local defaultTransparentRed = { r = 1, g = 0, b = 0, a = 0.5 }
local defaultTransparentBlue = { r = 0, g = 0, b = 1, a = 0.5 }
local defaultTransparentLightBlue = { r = 0.45, g = 0.62, b = 0.81, a = 0.5 }
local defaultTransparentGreen = { r = 0, g = 1, b = 0, a = 0.5 }
local defaultTransparentYellow = { r = 1, g = 1, b = 0, a = 0.5 }

local defaultCastbarColors = {
	cast = { r = 1, g = 1, b = 0, a = 1 },
	channel = { r = 0.5, g = 0.5, b = 1, a = 1 },
	mirror = { r = 0.75, g = 0.75, b = 0.1, a = 1 }, 
	background = { r = 0.5, g = 0.5, b = 0.5, a = 1 },
	interrupt = { r = 0.25, g = 0.25, b = 0.25, a = 1 },
}

local defaultGrid = {
	gridLines = 40, -- number of lines in overlay grid
	gridCenterColor = defaultRed, -- color of center lines in overlay grid
	gridLineColor = defaultGreen, -- color of other lines in overlay grid
	gridAlpha = 0.5, -- transparency of overlay grid
}

local defaultFont = "Interface\\AddOns\\Puffin\\Media\\Roboto-Medium.ttf"
local defaultFontSize = 12
local defaultFontColor = { r = 1, g = 1, b = 0, a = 1 }
local defaultFontFlags = "None"
local defaultBorder = [[Interface\BUTTONS\WHITE8X8.blp]]
local defaultBar = "Interface\\AddOns\\Puffin\\Media\\Gray"
local defaultBackground = "Interface\\AddOns\\Puffin\\Media\\Gray"

local defaultChatSettings = {
	font = defaultFont, fontSize = defaultFontSize, fontFlags = defaultFontFlags,
	width = 450, height = 350,
}

-- Defaults that are applied to all instances of each widget class
-- These are stored in a persistent database and can be changed using options panel
local widgetTemplates = {
	Widget_Backdrop = { -- superclass = "Widget"
		borderTexture = defaultBorder, -- this provides a single pixel black border around the bar
		borderWidth = 1, borderOffset = 1, borderColor = defaultBlack,
	},
	Widget_Panel = { -- superclass = "Widget_Backdrop"
		backdropTexture = "Interface\\AddOns\\Puffin\\Media\\White", -- this provides a panel that works with full range of colors and alpha settings
		backdropTile = 0, backdropInset = 1, backdropColor = defaultBlue,
	},
	Widget_Text = { -- superclass = "Widget"
		font = defaultFont, -- font to use for the text
		fsize = 14, -- font size for the text
		delta = 0, -- delta to apply to the font size
		shadow = true, -- enable a shadow behind the text
		outline = false, -- enable font "OUTLINE" flag
		thick = false, -- enable font "THICKOUTLINE" flag
		mono = false, -- enable font "MONOCHROME" flag
		wrap = false, -- enable wrap of text to multiple lines
		justifyH = "CENTER", -- horizontal text alignment
		justifyV = "MIDDLE", -- vertical text alignment
		color = defaultWhite, -- font color
	},
	Widget_Time = { -- superclass = "Widget_Text"
		timeFormat = 34,
	},
	Widget_Icon = { -- superclass = "Widget_Backdrop"
		iconSize = 16, texture = nil, texcoord = { 0.08, 0.92, 0.08, 0.92 }, -- trim icons to remove blizzard's edge design
	},
	Widget_Bar = { -- superclass = "Widget_Backdrop"
		fgTexture = defaultBar, fgColor = defaultGreen, fgAlpha = 1, sparkAlpha = 0.2,
		bgTexture = defaultBar, bgColor = defaultGreen, bgAlpha = 0.6,
		direction = false, fillBars = false,
	},
	Widget_Rectangle = { -- superclass = "Widget_Backdrop"
		texture = defaultBackground, color = defaultRed,
	},
	Widget_Unit = { -- superclass = "Widget"
	},
}

-- Layouts that describe groups of widgets used in views together
local widgetLayouts = {
	castbar_large = { -- a large castbar using a bar, left-side icon, and two text widgets
		[1] = {
			class = "Widget_Bar", -- base widget
			attributes = {
				width = 188, height = 12, depth = 8, bgColor = defaultLightGray, bgAlpha = 1,
			},
		},
		[2] = {
			class = "Widget_Icon", point = "LEFT", relative = "RIGHT", offsetX = 14, offsetY = 5,
			attributes = {
				width = 24, height = 24, depth = 1,
			},
		},
		[3] = {
			class = "Widget_Text", point = "TOPLEFT", relative = "BOTTOMLEFT", offsetX = 5, offsetY = -2,
			attributes = {
				width = 180, height = 20, depth = 12, justifyH = "LEFT", justifyV = "TOP", delta = -2,
			},
		},
		[4] = {
			class = "Widget_Time", point = "TOPLEFT", relative = "BOTTOMRIGHT", offsetX = 10, offsetY = -2,
			attributes = {
				width = 32, height = 20, depth = 14, justifyH = "CENTER", justifyV = "TOP", delta = -2,
			},
		},
		[5] = {
			class = "Widget_Text", point = "RIGHT", relative = "RIGHT", offsetX = -5, offsetY = 0,
			attributes = {
				width = 180, height = 24, depth = 12, justifyH = "RIGHT", color = defaultLight, delta = -3,
			},
		},
	},
	castbar_mirror = { -- a large castbar using a bar, left-side icon, and two text widgets
		[1] = {
			class = "Widget_Bar", -- base widget
			attributes = {
				width = 250, height = 24, depth = 4,
			},
		},
		[2] = {
			class = "Widget_Icon", point = "RIGHT", relative = "LEFT", offsetX = -3, offsetY = 0,
			attributes = {
				width = 24, height = 24, depth = 1,
			},
		},
		[3] = {
			class = "Widget_Text", point = "LEFT", relative = "LEFT", offsetX = 5, offsetY = 0,
			attributes = {
				width = 240, height = 24, depth = 8, justifyH = "LEFT",
			},
		},
		[4] = {
			class = "Widget_Time", point = "RIGHT", relative = "RIGHT", offsetX = -5, offsetY = 0,
			attributes = {
				width = 240, height = 24, depth = 10, justifyH = "RIGHT", delta = -2, timeFormat = 1,
			},
		},
	},
	castbar_small = { -- a smaller castbar using a bar, left-side icon, and a text widget for the label
		[1] = {
			class = "Widget_Bar", -- base widget
			attributes = {
				width = 250, height = 22, depth = 4,
			},
		},
		[2] = {
			class = "Widget_Icon", point = "RIGHT", relative = "LEFT", offsetX = -3, offsetY = 0,
			attributes = {
				width = 22, height = 22, depth = 1,
			},
		},
		[3] = {
			class = "Widget_Text", point = "LEFT", relative = "LEFT", offsetX = 5, offsetY = 0,
			attributes = {
				width = 220, height = 22, depth = 8, justifyH = "LEFT",
			},
		},
	},
	minimap = { -- border frame around the minimap
		[1] = {
			class = "Widget_Panel",  -- base widget
			attributes = {
				width = 0, height = 0, depth = 1,
				borderWidth = 1, borderOffset = 2, borderColor = defaultBlack, backdropColor = defaultLightGray
			},
		},
	},
	uf_player = { -- set of widgets for the player unit frame
		[1] = { -- main panel
			class = "Widget_Bar", -- health bar is base widget
			attributes = {
				width = 400, height = 16, depth = 2, fillBars = true, direction = false, bgColor = defaultTransparentLightGray,
				borderWidth = 1, borderOffset = 1, borderColor = defaultBlack,
			},
		},
		[2] = { -- background panel
			class = "Widget_Panel", point = "LEFT", relative = "LEFT", offsetX = 0, offsetY = 0,
			attributes = {
				width = 400, height = 16, depth = 1,
				borderWidth = 1, borderOffset = 3, borderColor = defaultBlack,
				backdropTexture = "Interface\\AddOns\\Puffin\\Media\\White", backdropColor = defaultLightGray,
			},
		},
		[3] = { -- name level race class text
			class = "Widget_Text", point = "TOPLEFT", relative = "TOPLEFT", offsetX = 4, offsetY = 16,
			attributes = {
				width = 360, height = 22, depth = 8, justifyH = "LEFT", justifyV = "TOP", fsize = 12,
			},
		},
		[4] = { -- raid target icon panel
			class = "Widget_Icon", point = "BOTTOM", relative = "TOPLEFT", offsetX = -4, offsetY = 2,
			attributes = {
				width = 16, height = 16, depth = 6, borderTexture = false, backdropTexture = false,
				texture = "Interface\\TargetingFrame\\UI-RaidTargetingIcons", texcoordIgnore = true,
			},
		},
		[5] = { -- mana/power bar is in inset panel
			class = "Widget_Bar", point = "TOPLEFT", relative = "BOTTOMLEFT", offsetX = 8, offsetY = 6,
			attributes = {
				width = 188, height = 12, depth = 4, fillBars = true, direction = false, bgColor = defaultLightGray, bgAlpha = 1,
				borderWidth = 1, borderOffset = 1, borderColor = defaultBlack,
			},
		},
		[6] = { -- pvp color square panel
			class = "Widget_Panel", point = "TOPRIGHT", relative = "TOPLEFT", offsetX = 4, offsetY = -13,
			attributes = {
				width = 6, height = 6, depth = 6,
				borderWidth = 1, borderOffset = 2, borderColor = defaultBlack, backdropColor = defaultRed,
				backdropTexture = "Interface\\AddOns\\Puffin\\Media\\Gray",
			},
		},
		[7] = { -- status/reaction color square panel
			class = "Widget_Panel", point = "TOPRIGHT", relative = "TOPLEFT", offsetX = -7, offsetY = -1,
			attributes = {
				width = 10, height = 14, depth = 2,
				borderWidth = 1, borderOffset = 2, borderColor = defaultBlack, backdropColor = defaultRed,
				backdropTexture = "Interface\\AddOns\\Puffin\\Media\\White",
			},
		},
	},
	uf_large = { -- set of widgets for the other main unit frames (e.g., target, focus)
		[1] = { -- main panel
			class = "Widget_Bar", -- health bar is base widget
			attributes = {
				width = 400, height = 16, depth = 1, fillBars = true, direction = false, bgColor = defaultTransparentLightGray,
				borderWidth = 1, borderOffset = 1, borderColor = defaultBlack,
			},
		},
		[2] = { -- background panel
			class = "Widget_Panel", point = "LEFT", relative = "LEFT", offsetX = 0, offsetY = 0,
			attributes = {
				width = 400, height = 16, depth = 1,
				borderWidth = 1, borderOffset = 3, borderColor = defaultBlack,
				backdropTexture = "Interface\\AddOns\\Puffin\\Media\\White", backdropColor = defaultLightGray,
			},
		},
		[3] = { -- name level race class text
			class = "Widget_Text", point = "TOPLEFT", relative = "TOPLEFT", offsetX = 4, offsetY = 16,
			attributes = {
				width = 360, height = 22, depth = 8, justifyH = "LEFT", justifyV = "TOP", fsize = 12,
			},
		},
		[4] = { -- raid target icon panel
			class = "Widget_Icon", point = "BOTTOM", relative = "TOPLEFT", offsetX = -4, offsetY = 2,
			attributes = {
				width = 16, height = 16, depth = 6, borderTexture = false, backdropTexture = false,
				texture = "Interface\\TargetingFrame\\UI-RaidTargetingIcons", texcoordIgnore = true,
			},
		},
		[5] = { -- mana/power bar is in inset panel
			class = "Widget_Bar", point = "TOPLEFT", relative = "BOTTOMLEFT", offsetX = 8, offsetY = 6,
			attributes = {
				width = 188, height = 12, depth = 4, fillBars = true, direction = false, bgColor = defaultLightGray, bgAlpha = 1,
				borderWidth = 1, borderOffset = 1, borderColor = defaultBlack,
			},
		},
		[6] = { -- pvp color square panel
			class = "Widget_Panel", point = "TOPRIGHT", relative = "TOPLEFT", offsetX = 4, offsetY = -13,
			attributes = {
				width = 6, height = 6, depth = 6,
				borderWidth = 1, borderOffset = 2, borderColor = defaultBlack, backdropColor = defaultRed,
				backdropTexture = "Interface\\AddOns\\Puffin\\Media\\Gray",
			},
		},
		[7] = { -- status/reaction color square panel
			class = "Widget_Panel", point = "TOPRIGHT", relative = "TOPLEFT", offsetX = -7, offsetY = -1,
			attributes = {
				width = 10, height = 14, depth = 1,
				borderWidth = 1, borderOffset = 2, borderColor = defaultBlack, backdropColor = defaultYellow,
				backdropTexture = "Interface\\AddOns\\Puffin\\Media\\Gray",
			},
		},
	},
	uf_small = { -- set of widgets for small-style unit frames (e.g., pet, targettarget)
		[1] = { -- main panel
			class = "Widget_Bar", -- health bar is base widget
			attributes = {
				width = 204, height = 16, depth = 1, fillBars = true, direction = false, bgColor = defaultTransparentLightGray,
				borderWidth = 1, borderOffset = 1, borderColor = defaultBlack,
			},
		},
		[2] = { -- background panel
			class = "Widget_Panel", point = "LEFT", relative = "LEFT", offsetX = 0, offsetY = 0,
			attributes = {
				width = 204, height = 16, depth = 1,
				borderWidth = 1, borderOffset = 3, borderColor = defaultBlack,
				backdropTexture = "Interface\\AddOns\\Puffin\\Media\\White", backdropColor = defaultLightGray,
			},
		},
		[3] = { -- name level race class text
			class = "Widget_Text", point = "TOPLEFT", relative = "TOPLEFT", offsetX = 4, offsetY = 16,
			attributes = {
				width = 360, height = 22, depth = 8, justifyH = "LEFT", justifyV = "TOP", fsize = 12,
			},
		},
		[4] = { -- raid target icon panel
			class = "Widget_Icon", point = "BOTTOM", relative = "TOPLEFT", offsetX = -4, offsetY = 2,
			attributes = {
				width = 16, height = 16, depth = 6, borderTexture = false, backdropTexture = false,
				texture = "Interface\\TargetingFrame\\UI-RaidTargetingIcons", texcoordIgnore = true,
			},
		},
		[5] = { -- mana/power bar is in inset panel
			class = "Widget_Bar", point = "TOPLEFT", relative = "BOTTOMLEFT", offsetX = 8, offsetY = 6,
			attributes = {
				width = 188, height = 12, depth = 4, fillBars = true, direction = false, bgColor = defaultLightGray, bgAlpha = 1,
				borderWidth = 1, borderOffset = 1, borderColor = defaultBlack,
			},
		},
		[6] = { -- pvp color square panel
			class = "Widget_Panel", point = "TOPRIGHT", relative = "TOPLEFT", offsetX = 4, offsetY = -13,
			attributes = {
				width = 6, height = 6, depth = 6,
				borderWidth = 1, borderOffset = 2, borderColor = defaultBlack, backdropColor = defaultRed,
				backdropTexture = "Interface\\AddOns\\Puffin\\Media\\Gray",
			},
		},
	},
	uf_minimal = { -- set of widgets for minimal unit frames (e.g., bosses)
		[1] = { -- main panel
			class = "Widget_Bar", -- health bar is base widget
			attributes = {
				width = 204, height = 16, depth = 1, fillBars = true, direction = false, bgColor = defaultTransparentLightGray,
				borderWidth = 1, borderOffset = 1, borderColor = defaultBlack,
			},
		},
		[2] = { -- background panel
			class = "Widget_Panel", point = "LEFT", relative = "LEFT", offsetX = 0, offsetY = 0,
			attributes = {
				width = 204, height = 16, depth = 1,
				borderWidth = 1, borderOffset = 3, borderColor = defaultBlack,
				backdropTexture = "Interface\\AddOns\\Puffin\\Media\\White", backdropColor = defaultLightGray,
			},
		},
		[3] = { -- name level race class text
			class = "Widget_Text", point = "TOPLEFT", relative = "TOPLEFT", offsetX = 4, offsetY = 16,
			attributes = {
				width = 360, height = 22, depth = 8, justifyH = "LEFT", justifyV = "TOP", fsize = 12,
			},
		},
		[4] = { -- raid target icon panel
			class = "Widget_Icon", point = "BOTTOM", relative = "TOPLEFT", offsetX = -4, offsetY = 2,
			attributes = {
				width = 16, height = 16, depth = 6, borderTexture = false, backdropTexture = false,
				texture = "Interface\\TargetingFrame\\UI-RaidTargetingIcons", texcoordIgnore = true,
			},
		},
	},
	resources = { -- set of widgets for displaying resources (e.g., combo points)
		[1] = { -- main panel is behind the other widgets and by default is transparent
			class = "Widget_Panel",
			attributes = {
				width = 188, height = 12, depth = 5, borderTexture = false, backdropTexture = false,
				panel_inset = { [3] = 28, [4] = 7, [5] = 5, [6] = 4 }, -- parameters for adapting to different numbers of resources
				panel_spacing = { [3] = 5, [4] = 5, [5] = 5, [6] = 5 },
				panel_width = { [3] = 40, [4] = 40, [5] = 32, [6] = 26 },
				panel_height = 17,
			},
		},
		[2] = { -- color square panel #1
			class = "Widget_Panel", point = "LEFT", relative = "LEFT", offsetX = 5, offsetY = 0,
			attributes = {
				width = 32, height = 8, depth = 6,
				borderWidth = 1, borderOffset = 1, borderColor = defaultBlack, backdropColor = defaultYellow,
				backdropTexture = "Interface\\AddOns\\Puffin\\Media\\White",
			},
		},
		[3] = { -- color square panel #2
			class = "Widget_Panel", point = "LEFT", relative = "LEFT", offsetX = 42, offsetY = 0,
			attributes = {
				width = 32, height = 8, depth = 6,
				borderWidth = 1, borderOffset = 1, borderColor = defaultBlack, backdropColor = defaultYellow,
				backdropTexture = "Interface\\AddOns\\Puffin\\Media\\White",
			},
		},
		[4] = { -- color square panel #3
			class = "Widget_Panel", point = "CENTER", relative = "CENTER", offsetX = 0, offsetY = 0,
			attributes = {
				width = 32, height = 8, depth = 6,
				borderWidth = 1, borderOffset = 1, borderColor = defaultBlack, backdropColor = defaultYellow,
				backdropTexture = "Interface\\AddOns\\Puffin\\Media\\White",
			},
		},
		[5] = { -- color square panel #4
			class = "Widget_Panel", point = "RIGHT", relative = "RIGHT", offsetX = -42, offsetY = 0,
			attributes = {
				width = 32, height = 8, depth = 6,
				borderWidth = 1, borderOffset = 1, borderColor = defaultBlack, backdropColor = defaultYellow,
				backdropTexture = "Interface\\AddOns\\Puffin\\Media\\White",
			},
		},
		[6] = { -- color square panel #5
			class = "Widget_Panel", point = "RIGHT", relative = "RIGHT", offsetX = -5, offsetY = 0,
			attributes = {
				width = 32, height = 8, depth = 6,
				borderWidth = 1, borderOffset = 1, borderColor = defaultBlack, backdropColor = defaultYellow,
				backdropTexture = "Interface\\AddOns\\Puffin\\Media\\White",
			},
		},
		[7] = { -- color square panel #6
			class = "Widget_Panel", point = "RIGHT", relative = "RIGHT", offsetX = -5, offsetY = 0,
			attributes = {
				width = 32, height = 8, depth = 6,
				borderWidth = 1, borderOffset = 1, borderColor = defaultBlack, backdropColor = defaultBlue,
				backdropTexture = "Interface\\AddOns\\Puffin\\Media\\White",
			},
		},
	},
}

-- View configurations that provide position, widgets, and data providers for each view
local viewConfigurations = {
	playerCastbar = {
		enabled = true, class = "View_Castbar", layout = "castbar_large", unit = "player",
		point = "TOPLEFT", relativeFrame = "Puffin_Player_UF", relative = "BOTTOMLEFT", offsetX = 204, offsetY = 6,
	},
	mirrorCastbar = {
		enabled = true, class = "View_Castbar", layout = "castbar_mirror", unit = "mirror",
		point = "TOP", relative = "TOP", offsetX = 0, offsetY = -100,
	},
	targetCastbar = {
		enabled = true, class = "View_Castbar", layout = "castbar_large", unit = "target",
		point = "TOPLEFT", relativeFrame = "Puffin_Target_UF", relative = "BOTTOMLEFT", offsetX = 204, offsetY = 6,
	},
	focusCastbar = {
		enabled = true, class = "View_Castbar", layout = "castbar_large", unit = "focus",
		point = "TOPLEFT", relativeFrame = "Puffin_Focus_UF", relative = "BOTTOMLEFT", offsetX = 204, offsetY = 6,
	},
	petCastbar = {
		enabled = true, class = "View_Castbar", layout = "castbar_small", unit = "pet",
		point = "TOPLEFT", relativeFrame = "Puffin_Pet_UF", relative = "BOTTOMLEFT", offsetX = 8, offsetY = 6,
	},
	minimap = {
		enabled = true, class = "View_Minimap", layout = "minimap", scale = 1,
		point = "BOTTOM", relative = "BOTTOM", offsetX = 700, offsetY = 45,
	},
	resources = {
		enabled = true, class = "View_Resources", layout = "resources",
		point = "TOPLEFT", relativeFrame = "Puffin_Player_UF", relative = "BOTTOMLEFT", offsetX = 204, offsetY = 6,
	},
	playerUnit = {
		enabled = true, class = "View_Unit", unit = "player", layout = "uf_player",
		point = "BOTTOM", relativeFrame = "Puffin_Player_UF", relative = "BOTTOM", offsetX = 0, offsetY = 0,
	},
	targetUnit = {
		enabled = true, class = "View_Unit", unit = "target", layout = "uf_large",
		point = "BOTTOM", relativeFrame = "Puffin_Target_UF", relative = "BOTTOM", offsetX = 0, offsetY = 0,
	},
	targettargetUnit = {
		enabled = true, class = "View_Unit", unit = "targettarget", layout = "uf_small",
		point = "BOTTOM", relativeFrame = "Puffin_TargetTarget_UF", relative = "BOTTOM", offsetX = 0, offsetY = 0,
	},
	focusUnit = {
		enabled = true, class = "View_Unit", unit = "focus", layout = "uf_large",
		point = "BOTTOM", relativeFrame = "Puffin_Focus_UF", relative = "BOTTOM", offsetX = 0, offsetY = 0,
	},
	focustargetUnit = {
		enabled = true, class = "View_Unit", unit = "focustarget", layout = "uf_small",
		point = "BOTTOM", relativeFrame = "Puffin_FocusTarget_UF", relative = "BOTTOM", offsetX = 0, offsetY = 0,
	},
	petUnit = {
		enabled = true, class = "View_Unit", unit = "pet", layout = "uf_small",
		point = "BOTTOM", relativeFrame = "Puffin_Pet_UF", relative = "BOTTOM", offsetX = 0, offsetY = 0,
	},
	boss1Unit = {
		enabled = true, class = "View_Unit", unit = "boss1", layout = "uf_minimal",
		point = "BOTTOM", relativeFrame = "Puffin_Boss1_UF", relative = "BOTTOM", offsetX = 0, offsetY = 0,
	},
	boss2Unit = {
		enabled = true, class = "View_Unit", unit = "boss2", layout = "uf_minimal",
		point = "BOTTOM", relativeFrame = "Puffin_Boss2_UF", relative = "BOTTOM", offsetX = 0, offsetY = 0,
	},
	boss3Unit = {
		enabled = true, class = "View_Unit", unit = "boss3", layout = "uf_minimal",
		point = "BOTTOM", relativeFrame = "Puffin_Boss3_UF", relative = "BOTTOM", offsetX = 0, offsetY = 0,
	},
	boss4Unit = {
		enabled = true, class = "View_Unit", unit = "boss4", layout = "uf_minimal",
		point = "BOTTOM", relativeFrame = "Puffin_Boss4_UF", relative = "BOTTOM", offsetX = 0, offsetY = 0,
	},
	boss5Unit = {
		enabled = true, class = "View_Unit", unit = "boss5", layout = "uf_minimal",
		point = "BOTTOM", relativeFrame = "Puffin_Boss5_UF", relative = "BOTTOM", offsetX = 0, offsetY = 0,
	},
}

local defaultViews = {
	"playerCastbar", "mirrorCastbar", "targetCastbar", "focusCastbar", "minimap", "resources",
	"playerUnit", "targetUnit", "targettargetUnit", "focusUnit", "focustargetUnit", "petUnit",
	"boss1Unit", "boss2Unit", "boss3Unit", "boss4Unit", "boss5Unit", 
}

-- Unit configurations that provide position and size for interactive frames that are associated with view unit frames
local unitConfigurations = {
	player = {
		enabled = true, frame = "Puffin_Player_UF", point = "BOTTOM", relative = "BOTTOM", offsetX = 0, offsetY = 185, width = 400, height = 16,
	},
	target = {
		enabled = true, frame = "Puffin_Target_UF", point = "BOTTOMLEFT", relative = "BOTTOM", offsetX = 372, offsetY = 550, width = 400, height = 16,
	},
	targettarget = {
		enabled = true, frame = "Puffin_TargetTarget_UF", point = "BOTTOMLEFT", relative = "BOTTOM", offsetX = 372, offsetY = 500, width = 204, height = 16,
	},
	focus = {
		enabled = true, frame = "Puffin_Focus_UF", point = "BOTTOMLEFT", relative = "BOTTOM", offsetX = 372, offsetY = 360, width = 400, height = 16,
	},
	focustarget = {
		enabled = true, frame = "Puffin_FocusTarget_UF", point = "BOTTOMLEFT", relative = "BOTTOM", offsetX = 372, offsetY = 310, width = 204, height = 16,
	},
	pet = {
		enabled = true, frame = "Puffin_Pet_UF", point = "BOTTOMLEFT", relative = "BOTTOM", offsetX = -642, offsetY = 100, width = 204, height = 16,
	},
	boss1 = {
		enabled = true, frame = "Puffin_Boss1_UF", point = "TOPLEFT", relative = "TOPLEFT", offsetX = 100, offsetY = -100, width = 204, height = 16,
	},
	boss2 = {
		enabled = true, frame = "Puffin_Boss2_UF", point = "TOPLEFT", relative = "TOPLEFT", offsetX = 100, offsetY = -150, width = 204, height = 16,
	},
	boss3 = {
		enabled = true, frame = "Puffin_Boss3_UF", point = "TOPLEFT", relative = "TOPLEFT", offsetX = 100, offsetY = -200, width = 204, height = 16,
	},
	boss4 = {
		enabled = true, frame = "Puffin_Boss4_UF", point = "TOPLEFT", relative = "TOPLEFT", offsetX = 100, offsetY = -250, width = 204, height = 16,
	},
	boss5 = {
		enabled = true, frame = "Puffin_Boss5_UF", point = "TOPLEFT", relative = "TOPLEFT", offsetX = 100, offsetY = -300, width = 204, height = 16,
	},
}

local defaultUnits = { "player", "pet", "target", "targettarget", "focus", "focustarget", "boss1", "boss2", "boss3", "boss4", "boss5" }

-- Add default and layout configuration data to the database then create default views
local function onInitialize()
	if override or not DB.IsDatabase("defaults") then
		DB.AllocateDatabase("defaults", true, true)
		DB.SetAttribute("defaults", "colors", "black", defaultBlack)
		DB.SetAttribute("defaults", "colors", "dark", defaultDark)
		DB.SetAttribute("defaults", "colors", "gray", defaultGray)
		DB.SetAttribute("defaults", "colors", "lightGray", defaultLightGray)
		DB.SetAttribute("defaults", "colors", "light", defaultLight)
		DB.SetAttribute("defaults", "colors", "white", defaultWhite)
		DB.SetAttribute("defaults", "colors", "red", defaultRed)
		DB.SetAttribute("defaults", "colors", "blue", defaultBlue)
		DB.SetAttribute("defaults", "colors", "lightBlue", defaultLightBlue)
		DB.SetAttribute("defaults", "colors", "green", defaultGreen)
		DB.SetAttribute("defaults", "colors", "yellow", defaultYellow)
		DB.SetAttribute("defaults", "colors", "orange", defaultOrange)
		DB.SetAttribute("defaults", "colors", "transparentBlack", defaultTransparentBlack)
		DB.SetAttribute("defaults", "colors", "transparentDark", defaultTransparentDark)
		DB.SetAttribute("defaults", "colors", "transparentGray", defaultTransparentGray)
		DB.SetAttribute("defaults", "colors", "transparentLightGray", defaultTransparentLightGray)
		DB.SetAttribute("defaults", "colors", "transparentLight", defaultTransparentLight)
		DB.SetAttribute("defaults", "colors", "transparentWhite", defaultTransparentWhite)
		DB.SetAttribute("defaults", "colors", "transparentRed", defaultTransparentRed)
		DB.SetAttribute("defaults", "colors", "transparentBlue", defaultTransparentBlue)
		DB.SetAttribute("defaults", "colors", "transparentLightBlue", defaultTransparentLightBlue)
		DB.SetAttribute("defaults", "colors", "transparentGreen", defaultTransparentGreen)
		DB.SetAttribute("defaults", "colors", "transparentYellow", defaultTransparentYellow)
		DB.SetRecord("defaults", "castbar", defaultCastbarColors)
		DB.SetRecord("defaults", "grid", defaultGrid)
		DB.SetRecord("defaults", "chat", defaultChatSettings)
		DB.SetAttribute("defaults", "Masque", "enable", true)
		DB.SetAttribute("defaults", "Masque", "color", defaultGray)
		DB.SetAttribute("defaults", "configuration", "mode", configurationMode)
		DB.SetAttribute("defaults", "configuration", "updateTarget", 0.1)
		DB.SetAttribute("defaults", "media", "font", defaultFont)
		DB.SetAttribute("defaults", "media", "fontSize", defaultFontSize)
		DB.SetAttribute("defaults", "media", "fontColor", defaultFontColor)
		DB.SetAttribute("defaults", "media", "fontFlags", defaultFontFlags)
		DB.SetAttribute("defaults", "media", "border", defaultBorder)
		DB.SetAttribute("defaults", "media", "bar", defaultBar)
		DB.SetAttribute("defaults", "media", "background", defaultBackground)
	end

	if override or not DB.IsDatabase("templates") then -- create persistent database of widget templates
		DB.AllocateDatabase("templates", true, true)
		for k, v in pairs(widgetTemplates) do DB.SetRecord("templates", k, v, true) end
	end

	if override or not DB.IsDatabase("layouts") then -- create persistent database of widget layouts
		DB.AllocateDatabase("layouts", true, true)
		for k, v in pairs(widgetLayouts) do DB.SetRecord("layouts", k, v, true) end
	end
	
	if override or not DB.IsDatabase("views") then -- create persistent database of view descriptions
		DB.AllocateDatabase("views", true, true)
		for k, v in pairs(viewConfigurations) do DB.SetRecord("views", k, v, true) end
		DB.SetRecord("views", "default", defaultViews)
	end

	if override or not DB.IsDatabase("units") then -- create persistent database of view descriptions
		DB.AllocateDatabase("units", true, true)
		for k, v in pairs(unitConfigurations) do DB.SetRecord("units", k, v, true) end
		DB.SetRecord("units", "default", defaultUnits)
	end

	local updateTarget = DB.GetAttribute("defaults", "configuration", "updateTarget")
	if updateTarget then Puffin.updateTarget = updateTarget end
	
	Puffin.Callback("Puffin_Configure") -- trigger configuration in all modules
end

Puffin.RegisterCallback("Puffin_Initialize", onInitialize)
